library(lme4)
library(coda)
library(runjags)
library(MASS)
library(bayesm)

# set working directory to location of this file
setwd('~')



# SIMULATE DATA

# set up observational units

r1.cluster.n <- 50
n_per_cluster1 <- 10
n <- r1.cluster.n * n_per_cluster1

n_per_cluster2 <- 20
r2.cluster.n <- n/n_per_cluster2

r1.cluster.i <- gl(n=r1.cluster.n, k=n_per_cluster1)
r2.cluster.i <- gl(n=r2.cluster.n, k=n_per_cluster2)


P <- 1
Q1 <- 3
Q2 <- 2


# fixed part

B.fixed <- c(90)

x1.fixed <- rnorm(n, 0, 1); x1.fixed <- (x1.fixed-mean(x1.fixed))/sd(x1.fixed)
X.fixed <- model.matrix(~ x1.fixed - 1)

fixed <- X.fixed %*% B.fixed


# random part #1

B.random1 <- c(100, 80, 70)

x1.random1 <- rnorm(n, 0, 1); x1.random1 <- (x1.random1-mean(x1.random1))/sd(x1.random1)
x2.random1 <- rnorm(n, 0, 1); x2.random1 <- (x2.random1-mean(x2.random1))/sd(x2.random1)


rho.B1 <- matrix(c(
    30,  0.1,  0.3,
   0.1,   50,  0.4,
   0.3,  0.4,   70
), Q1, Q1)


X.random1 <- model.matrix(~ r1.cluster.i*x1.random1 + r1.cluster.i*x2.random1 -x1.random1-x2.random1-1)
B.random1 <- mvrnorm(n=r1.cluster.n, mu=B.random1, Sigma=rho.B1)

random1 <- X.random1 %*% as.vector(B.random1)


# random part #2

B.random2 <- c(0, 60)

x1.random2 <- rnorm(n, 0, 1); x1.random2 <- (x1.random2-mean(x1.random2))/sd(x1.random2)


rho.B2 <- matrix(c(
    10,  0.0,
   0.0,   10
), Q2, Q2)


X.random2 <- model.matrix(~ r2.cluster.i*x1.random2 -x1.random2-1)
B.random2 <- mvrnorm(n=r2.cluster.n, mu=B.random2, Sigma=rho.B2)

random2 <- X.random2 %*% as.vector(B.random2)


# construct y

y.hat <- fixed + random1 + random2

y.noise <- 80
y <- y.hat + rnorm(n, 0, sqrt(y.noise))

d <- data.frame(r1.cluster.i, r2.cluster.i, x1 = x1.fixed, x2 = x1.random1, x3 = x2.random1, x4 = x1.random2, y)



# ANALYSIS

X.fixed <-  as.matrix(model.frame(~ x1 + x2 + x3 + x4, data = d))
X.random1 <- as.matrix(model.frame(~ x2 + x3, data = d))
X.random2 <- as.matrix(model.frame(~ x4, data = d))

summary(lmer(y ~ X.fixed + (1+X.random1 | r1.cluster.i) + (1+X.random2 | r2.cluster.i), data=d))



# JAGS

X.fixed <-  model.matrix(~ x1 + x2 + x3 + x4, data = d)
X.random1 <- model.matrix(~ x2 + x3, data = d)
X.random2 <- model.matrix(~ x4, data = d)

r1.cluster.i <- as.numeric(as.factor(d$r1.cluster.i))
r2.cluster.i <- as.numeric(as.factor(d$r2.cluster.i))

y <- d$y


d_jags <- list (
		n = dim(d)[1], 
		r1.cluster.n = length(unique(r1.cluster.i)), 
		r1.cluster.i = r1.cluster.i,
		r2.cluster.n = length(unique(r2.cluster.i)), 
		r2.cluster.i = r2.cluster.i,
		X.fixed = X.fixed,
		X.random1 = X.random1,
		X.random2 = X.random2,
		P = ncol(X.fixed),
		Q1 = ncol(X.random1),
		Q2 = ncol(X.random2),
		y = y,
		W1 = diag(dim(X.random1)[2]),
		W2 = diag(dim(X.random2)[2])
)

inits1 <- list(
	y.noise = runif(1, 0, 100),
	B.fixed = rnorm(ncol(X.fixed), 0, 100),
	xi1 = runif(ncol(X.random1)),
	xi2 = runif(ncol(X.random2)),
	B.raw1 = array(rnorm(r1.cluster.n*ncol(X.random1)),c(r1.cluster.n,ncol(X.random1))),
	B.raw2 = array(rnorm(r2.cluster.n*ncol(X.random2)),c(r2.cluster.n,ncol(X.random2))),
	Tau.B.raw1 = rwishart(ncol(X.random1)+1,diag(ncol(X.random1)))$W, 
	Tau.B.raw2 = rwishart(ncol(X.random2)+1,diag(ncol(X.random2)))$W, 
	.RNG.name = "base::Super-Duper"
)
inits2 <- list(
	y.noise = runif(1, 0, 100),
	B.fixed = rnorm(ncol(X.fixed), 0, 100),
	xi1 = runif(ncol(X.random1)),
	xi2 = runif(ncol(X.random2)),
	B.raw1 = array(rnorm(r1.cluster.n*ncol(X.random1)),c(r1.cluster.n,ncol(X.random1))),
	B.raw2 = array(rnorm(r2.cluster.n*ncol(X.random2)),c(r2.cluster.n,ncol(X.random2))),
	Tau.B.raw1 = rwishart(ncol(X.random1)+1,diag(ncol(X.random1)))$W, 
	Tau.B.raw2 = rwishart(ncol(X.random2)+1,diag(ncol(X.random2)))$W, 
	.RNG.name = "base::Super-Duper"
)
inits3 <- list(
	y.noise = runif(1, 0, 100),
	B.fixed = rnorm(ncol(X.fixed), 0, 100),
	xi1 = runif(ncol(X.random1)),
	xi2 = runif(ncol(X.random2)),
	B.raw1 = array(rnorm(r1.cluster.n*ncol(X.random1)),c(r1.cluster.n,ncol(X.random1))),
	B.raw2 = array(rnorm(r2.cluster.n*ncol(X.random2)),c(r2.cluster.n,ncol(X.random2))),
	Tau.B.raw1 = rwishart(ncol(X.random1)+1,diag(ncol(X.random1)))$W, 
	Tau.B.raw2 = rwishart(ncol(X.random2)+1,diag(ncol(X.random2)))$W, 
	.RNG.name = "base::Super-Duper"
)


set.seed(123)
iterations <- 1000 * 2
jags <- run.jags(
		'JAGS/linear 6 - JAGS (scaled inverse wishart - non-nested random effects).R', data = d_jags,
		monitor=c('B.fixed', 'sigma.B1', 'rho.B1', 'sigma.B2', 'rho.B2', 'y.noise'), 
		n.chains=2, method='parallel', inits=list(inits1,inits2),   
		sample = iterations,
		burnin = round(iterations/4),
		adapt = round(iterations/10)
)

jags_summary <-	print(jags)
rownames(jags_summary)[grepl('B.fixed', rownames(jags_summary))]  <- colnames(X.fixed)
rownames(jags_summary)[grepl('sigma.B1', rownames(jags_summary))] <- paste('SD', colnames(X.random1), sep='_')
rownames(jags_summary)[grepl('sigma.B2', rownames(jags_summary))] <- paste('SD', colnames(X.random2), sep='_')

jags_summary